------------------------------
--      Are you local?      --
------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Postal")

---------------------------------
--      Addon Declaration      --
---------------------------------

Postal = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceModuleCore-2.0", "AceConsole-2.0", "AceDB-2.0")

Postal.revision = tonumber(string.sub("$Revision: 25567 $", 12, -3))

Postal:SetModuleMixins("AceEvent-2.0", "AceHook-2.1")

Postal.defaults = {}

Postal.consoleOptions = {
	type = "group",
	handler = Postal,
	args = {},
}

Postal:RegisterDB("PostalDB", "PostalDBPerChar")
Postal:RegisterDefaults("profile", Postal.defaults)
Postal:RegisterChatCommand({"/Postal", "/Postal"}, Postal.consoleOptions )

------------------------------
--      Initialization      --
------------------------------

function Postal:OnInitialize()
	if not self.version then self.version = GetAddOnMetadata("Postal", "Version") end
	local rev = self.revision
	for name, module in self:IterateModules() do
		rev = math.max(rev, module.revision)
	end
	self.version = (self.version or "1.0").. " |cffff8888r"..rev.."|r"
end

function Postal:OnEnable()
end

function Postal:OnDisable()
	-- Unconditionally disable all modules, disregarding module.shouldEnable
	local name, module = nil, nil
	for name, module in self:IterateModules() do
		self:ToggleModuleActive(module, false)
	end
end

--------------------------------
--      Module Prototype      --
--------------------------------

Postal.modulePrototype.core = Postal
Postal.modulePrototype.revision = 1

function Postal.modulePrototype:OnInitialize()
	-- Unconditionally register, this shouldn't happen from any other place
	-- anyway.
	self.core:RegisterModule(self.name, self)
end

function Postal.modulePrototype:IsRegistered()
	return self.registered
end

-------------------------------
--      Module Handling      --
-------------------------------

function Postal:RegisterModule(name, module)

	if module:IsRegistered() then
		error(string.format("%q is already registered.", name))
		return
	end

	if module.db and module.RegisterDefaults and type(module.RegisterDefaults) == "function" then
		module:RegisterDefaults("profile", opts or module.defaults or {})
	else
		self:RegisterDefaults(name, "profile", opts or module.defaults or {})
	end

	if not module.db then module.db = self:AcquireDBNamespace(name) end

	if module.consoleOptions then
		local m = module.consoleCmd or name
		-- if the consoleoption already exists we merge in the data otherwise we create a new option
		if self.consoleOptions.args[m] then
			for k,v in pairs(module.consoleOptions.args) do self.consoleOptions.args[m].args[k] = v end
		else
			self.consoleOptions.args[m] = module.consoleOptions
		end
	end

	module.registered = true
	if module.OnRegister and type(module.OnRegister) == "function" then
		module:OnRegister()
	end
end

